<?php
/**
 *      
 *
 *  -  ,    .
 *
 * @param   string  $file            
 * @param   string  $real_name    
 *
 * @author  Victor Grischenko <victor.grischenko@zveno.com.ua>
 * @package Utils
 * @since   07.06.2006
 */



/**
 *   
 *
 *    . 
 *  .
 *
 * @param   string   $fname          
 *                                 
 * @param   string   $realname      
 * @param   string   $mime        MIME- 
 * @param   bool     $bAttachment     
 * @return  bool                   true    
 *                                false .
 */
function download_partial( $fname, $realname, $mime, $bAttachment ) {

    if ( $bAttachment ) {
        $hdr_download = array("Content-Disposition", "attachment; filename=".cfPage::url($realname));
    }
    else {
        $hdr_download = array();
    }

    if ( !$fd = fopen($fname, "rb") ) {
        CCore::error_push(CError::create("403", __FILE__, __LINE__));
        return false;
    }

    if ( isset($_SERVER["HTTP_RANGE"]) ) {
        $range = $_SERVER["HTTP_RANGE"];
        $range = str_replace("bytes=", "", $range);
        $range = str_replace("-", "", $range);
        if ( $range ) {
            fseek($fd, $range);
        }
    }

    if ( isset($range) ) {
        header("HTTP/1.1 206 Partial Content");
    }
    else {
        header("HTTP/1.1 200 OK");
        $range=0;
    }

    $fsize    = filesize($fname);
    $modified = filemtime($fname);

    //  
    CCore::update_last_modified($modified);

    $oPage = cfPage::create();

    $oPage->addHeader("Content-Type", $mime);
    $oPage->addHeader("Content-Length", ($fsize-$range));
    $oPage->addHeader("Content-Range", "bytes ".$range."-".($fsize -1)."/".$fsize);

    if ( $hdr_download ) {
        $oPage->addHeader($hdr_download[0], $hdr_download[1]);
    }

    //   
    if ( !empty($_CONFIG["webface.cache.client"]) ) {
        //   
        $oPage->processCacheHeaders( md5($modified) );
    }

    $oPage->sendHeaders();


    $readed = $range;
    while (!feof($fd) || $readed<=$fsize) {

        set_time_limit(20);
        echo fread($fd, 1024);
        $readed += 1024;

        if ( connection_aborted() ) {
            //   
            break;
        }
    }

    fclose($fd);

    return true;

} // end of function download_parted()


// 
require($_SERVER["DOCUMENT_ROOT"]."/etc/init.php");
require($_CONFIG["general.libdir"]."/cutils.php");
require($_CONFIG["general.coredir"]."/centity.php");
require($CFPARAMS["basedir"]."/interface/view/cfpage.class.php");

do {

    $entid    = empty($_GET["entid"])    ? 0  : (int)$_GET["entid"];
    $field    = empty($_GET["field"])    ? "" : trim($_GET["field"]);
    $realname = empty($_GET["realname"]) ? "" : trim($_GET["realname"]);



    //   
    if ( !$oEnt = CEntity::create($entid) ) {
        CError::raise("Entity creation error", __FILE__, __LINE__);
        break;
    }

    if ( empty($oEnt->etype->fields_name[$field]) ) {
        CError::raise("Unknown field", __FILE__, __LINE__);
        break;
    }

    if ( $oEnt->etype->fields_name[$field]->params["field_type"] != "image" && $oEnt->etype->fields_name[$field]->params["field_type"] != "file" ) {
        CError::raise("Unsupported field type", __FILE__, __LINE__);
        break;
    }

    $fname    = $_CONFIG["general.uploaddir"]."/".$oEnt->params[$field];
    $realname = $oEnt->params[$field."_realname"];



    //  MIME-
    $arMimes = parse_ini_file($_CONFIG["general.etcdir"]."/mimes.ini", true);



    if ( !file_exists($fname) ) {
        CError::raise("404", __FILE__, __LINE__);
        break;
    }

    // -
    if (($ext = CUtils::get_file_ext($realname)) && isset($arMimes["inline"][$ext])) {
        //  ,    
        $mime        = $arMimes["inline"][$ext];
        $bAttachment = false;

    }
    else {
        //   ,  ,     

        if ($tmp_filename = iconv("UTF-8", "CP1251", $realname)) {
            $realname = $tmp_filename;
        }

        $mime        = empty($arMimes["attachment"][$ext]) ? "application/octet-stream" : $arMimes["attachment"][$ext];
        $bAttachment = true;
    }

    //   
    if ( !download_partial( $fname, $realname, $mime, $bAttachment ) ) {
        break;
    }

    //   
    exit;

} while (false);

//  
include($_CONFIG["general.basedir"]."/bin/error.php");
?>